package gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.*;

/**
 * The toolbar that enables the user to specify the input to the simulation.
 */
public class InputToolBar extends JToolBar {

    private SimulatorTab simulatorTab;
    private JTextField inputField;

    /**
     * Creates a new input toolbar.
     *
     * @param s The simulator tab this toolbar is for.
     */
    public InputToolBar(SimulatorTab s) {

        this.simulatorTab = s;

        setFloatable(false);

        add(new JLabel("Input:"));
        inputField = new JTextField(15);
        add(inputField);

        add(new JButton("Start simulation") {
            {
                addActionListener(new ActionListener() {

                    @Override
                    public void actionPerformed(ActionEvent arg0) {

                        // place the input on the tape
                        ArrayList<Character> inputArray = new ArrayList<>();
                        String input = inputField.getText();
                        for (int i = 0; i < input.length(); i++) {

                            if (input.charAt(i) == ' ' 
                                    || input.charAt(i) == '#') {
                                inputArray.add(null);
                            } else {
                                inputArray.add(input.charAt(i));
                            }
                        }
                        simulatorTab.getTuringMachine().getTape().placeInput(inputArray);

                        // go to the simulation mode
                        simulatorTab.setMode(SimulatorTab.SimulatorMode.SIMULATING);
                    }
                });
            }
        });
    }
}
