package gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTabbedPane;
import javax.swing.filechooser.FileNameExtensionFilter;

public class MenuBar extends JMenuBar implements ActionListener {

    /**
     * The menu options on the menu bar.
     */
    private JMenu fileMenu;

    /**
     * File chooser used when opening or saving files.
     */
    private JFileChooser fileChooser;

    /**
     * The options in the file menu bar option.
     */
    private JMenuItem openFile, saveFile, exit;

    /**
     * The tabbed pane containing the tabs.
     */
    private final JTabbedPane tabs;

    /**
     * The editor tab.
     */
    private EditorTab editorTab;

    /**
     * Constructor of the MenuBar class
     *
     * @param et the editor tab with the editor
     */
    MenuBar(JTabbedPane tabs, EditorTab editorTab) {

        this.tabs = tabs;
        this.fileChooser = new JFileChooser();
        this.fileChooser.setAcceptAllFileFilterUsed(false);
        this.fileChooser.addChoosableFileFilter(
                new FileNameExtensionFilter(".txt", "txt"));
        this.fileChooser.addChoosableFileFilter(
                new FileNameExtensionFilter(".ctm", "ctm"));

        // get the editor tab
        this.editorTab = editorTab;

        // Create the file menu bar option.
        this.fileMenu = new JMenu("File");
        this.openFile = new JMenuItem("Open file");
        this.openFile.addActionListener(this);

        this.fileMenu.add(openFile);
        this.saveFile = new JMenuItem("Save file");
        this.saveFile.addActionListener(this);
        this.fileMenu.add(saveFile);

        this.exit = new JMenuItem("Exit");
        this.exit.addActionListener(this);
        this.fileMenu.add(exit);

        // Add all the menu bar options to the menu bar.
        add(fileMenu);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        if (e.getSource().equals(openFile)) {
            openFile();
        } else if (e.getSource().equals(saveFile)) {
            saveFile();
        } else if (e.getSource().equals(exit)) {
            System.exit(0);
        }
    }
    
    /**
     * Open a file and load it into the editor tab through the file chooser
     * dialog.
     */
    private void openFile() {

        if (fileChooser.showOpenDialog(tabs) == JFileChooser.APPROVE_OPTION) {
            File file = fileChooser.getSelectedFile();
            try {
                Scanner scanner = new Scanner(file);
                String inputText = "";
                while (scanner.hasNextLine()) {
                    inputText += scanner.nextLine() + "\n";
                }
                editorTab.setProgramText(inputText);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        setTabForFileChooser();
    }
    
    /**
     * Save a file in either .txt or .ctm format through a chooser dialog.
     */
    private void saveFile() {

        if (fileChooser.showSaveDialog(tabs) == JFileChooser.APPROVE_OPTION) {

            try {
                
                String path = fileChooser.getSelectedFile().getAbsolutePath();
                String extension = fileChooser.getFileFilter().getDescription();
                File outputFile = new File(path + extension);
                FileOutputStream fos = new FileOutputStream(outputFile);
                BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(fos));
		String[] outputArray = editorTab.getProgramText().split("\\n");
                
                for (String line : outputArray) {
                    bw.write(line);
                    bw.newLine();
                }
                
                bw.close();
            } catch (IOException ex) {
                Logger.getLogger(MenuBar.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }
    
    /**
     * After opening a file we switch the editor tab.
     */
    private void setTabForFileChooser() {
        if (tabs.getSelectedIndex() == 1) {
            tabs.setSelectedIndex(0);
        }
    }
}
