package gui;

import model.Transition;
import model.TuringProgram;

import java.util.Scanner;

/**
 * This class implements a static method for parsing Turing programs.
 */
public class Parser {

    /**
     * Parses the given program.
     *
     * @param program The program to parse, as a string.
     * @return The resulting Turing program.
     * @throws ParseException If there was a problem while parsing, for example
     * a syntax error in the input.
     */
    public static TuringProgram parseProgram(String program) throws ParseException {

        TuringProgram tp = new TuringProgram();
        String tStr = "";
        Scanner s = null;
        int lineNumber = 0;

        // always add the initial state q0
        tp.addState("q0");

        try {
            s = new Scanner(program);

            while (s.hasNextLine()) {

                //get next line from input
                tStr = s.nextLine();
                lineNumber++;

                if (tStr.isEmpty()) {
                    //break out of current iteration and continue with the next one if its an empty line
                    continue;
                } else if (tStr.substring(0, 2).equals("%%")
                        || tStr.substring(0, 2).equals("//")) {
                    //break out of current iteration and continue with the next one if its a comment
                    continue;
                }

                //add transition, increment the number of the transitions by 1
                tp.addTransition(splitNewNotation(tStr));
            }
        } catch (Exception e) {
            //error while adding transition, fauly transition is printed
            throw new ParseException("Transition \"" + tStr + "\" is not valid", lineNumber);
        } finally {
            if (s != null) {
                s.close();
            }
        }

        return tp;

    }

    ///////// TODO TODO TODO
    private static Transition splitNewNotation(String transition) {

        String[] split = transition.split("\\s+");
        String[] tape = split[1].split(",");
        String[] tapeChars = tape[0].split("/");
        String currentState = split[0];
        char readTape = stringToChar(tapeChars[0]);
        String nextState = split[2];
        char writeTape = stringToChar(tapeChars[1]);
        String tapeHead = tape[1];

        Transition t = new Transition(currentState, nextState, readTape, writeTape, tapeHead);
        //Transition t = new Transition("q0", 'a', 'b', "R", "q1", 0);

        return t;
    }

    private static char stringToChar(String s) {

        char[] charArray = s.toCharArray();
        char ch = 0;

        for (char c : charArray) {
            ch += c;
        }

        return ch;
    }
}
